@@ -16,6 +16,8 @@
 #include "gmain.h"
 #include "cmd.h"
 #include "boxes.h"
+#include "profile.h"
+#include "setup.h"
 #include "panelize.h"
 #include "gcmd.h"
 #include "dialog.h"
@@ -30,6 +32,7 @@ static enum {
 	SORT_CHANGE,
 	SORT_SIZE
 } SortOrderCode;
+static char *panelize_section = "Panelize";
 
 void
 gnome_listing_cmd (GtkWidget *widget, WPanel *panel)
@@ -299,57 +302,232 @@ gnome_sort_cmd (GtkWidget *widget, WPanel *panel)
 	}
 	gtk_widget_destroy (sort_box);
 }
+typedef struct ep_dlg_data {
+	GtkWidget *ep_dlg;
+	GtkWidget *clist;
+	GtkWidget *entry;
+	GtkWidget *add_button;
+	GtkWidget *remove_button;
+	gboolean setting_text;
+	gint selected; /* if this is -1 then nothing is selected, otherwise, it's the row selected */
+} ep_dlg_data;
+
+static gchar *
+get_nickname (gchar *text)
+{
+	GtkWidget *dlg;
+	GtkWidget *entry;
+	GtkWidget *label;
+	gchar *retval = NULL;
+
+	dlg = gnome_dialog_new (_("Enter name."), GNOME_STOCK_BUTTON_OK, 
+				GNOME_STOCK_BUTTON_CANCEL, NULL);
+	entry = gtk_entry_new ();
+	if (text)
+		gtk_entry_set_text (GTK_ENTRY (entry), text);
+	label = gtk_label_new (_("Enter label for command:"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
+			    label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
+			    entry, FALSE, FALSE, 0);
+	gtk_widget_show_all (GNOME_DIALOG (dlg)->vbox);
+	switch (gnome_dialog_run (GNOME_DIALOG (dlg))) {
+	case 0:
+		retval = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+		break;
+	case 1:
+	default:
+	}
+	gtk_widget_destroy (dlg);
+	return retval;	
+}
+static void
+ep_add_callback (GtkWidget *widget, ep_dlg_data *data)
+{
+	gint i;
+	gchar *insert_tab[1];
+
+	insert_tab[0] = get_nickname (NULL);
+	if (insert_tab[0] == NULL)
+		return;
+	i = gtk_clist_append (GTK_CLIST (data->clist), insert_tab);
+	gtk_clist_set_row_data (GTK_CLIST (data->clist), i,
+				g_strdup (gtk_entry_get_text (GTK_ENTRY (data->entry))));
+	g_free (insert_tab [0]);
+	data->selected = -1;
+	gtk_widget_set_sensitive (data->add_button, FALSE);
+	gtk_entry_set_text (GTK_ENTRY (data->entry), "");
+}
+static void
+ep_remove_callback (GtkWidget *widget, ep_dlg_data *data)
+{
+	if (data->selected > -1) {
+		g_free (gtk_clist_get_row_data (GTK_CLIST (data->clist), data->selected));
+		gtk_clist_remove (GTK_CLIST (data->clist), data->selected);
+		data->selected = -1;
+		gtk_entry_set_text (GTK_ENTRY (data->entry), "");
+	}
+	gtk_widget_set_sensitive (data->remove_button, FALSE);
+}
+
+static void
+ep_select_callback (GtkWidget *widget,
+		    gint row,
+		    gint column,
+		    GdkEventButton *event,
+		    ep_dlg_data *data)
+{
+	if (event && event->type == GDK_2BUTTON_PRESS) {
+		gchar *nick;
+
+		gtk_clist_get_text (GTK_CLIST (widget), row, 0, &nick);
+		/* ugly but correct... (: */
+		nick = get_nickname (nick);
+		gtk_clist_set_text (GTK_CLIST (data->clist), row, 0, nick);
+		gtk_clist_select_row (GTK_CLIST (data->clist), row, 0);
+	} else {
+		data->setting_text = TRUE;
+		gtk_entry_set_text (GTK_ENTRY (data->entry),
+				    (gchar *) gtk_clist_get_row_data (GTK_CLIST (widget), row));
+		data->setting_text = FALSE;
+		data->selected = row;
+		gtk_widget_set_sensitive (data->remove_button, TRUE);
+		gtk_widget_set_sensitive (data->add_button, FALSE);
+	}
+}
+static void
+ep_text_changed_callback (GtkWidget *widget, ep_dlg_data *data)
+{
+	if (data->setting_text)
+		/* we don't want to deselect text if we just clicked on something */
+		return;
+	if (data->selected > -1) {
+		gtk_clist_unselect_row (GTK_CLIST (data->clist), data->selected, 0);
+		data->selected = -1;
+	}
+	gtk_widget_set_sensitive (data->remove_button, FALSE);
+	gtk_widget_set_sensitive (data->add_button, TRUE);
+}
+static void
+load_settings (GtkCList *clist)
+{
+	gchar *insert_tab[1];
+	void *profile_keys;
+	gchar *key, *value;
+	gint i = 0;
+
+	profile_keys = profile_init_iterator (panelize_section, profile_name);
+    
+	if (!profile_keys){
+		insert_tab[0] = _("Find all core files");
+		i = gtk_clist_insert (clist, i, insert_tab);
+		gtk_clist_set_row_data (clist, i, g_strdup ("find / -name core"));
+		insert_tab[0] = _("Find rejects after patching");
+		i = gtk_clist_insert (clist, i, insert_tab);
+		gtk_clist_set_row_data (clist, i, g_strdup ("find . -name \\*.rej -print"));
+	} else {
+		while (profile_keys) {
+			profile_keys = profile_iterator_next (profile_keys, &key, &value);
+			insert_tab[0] = key;
+			i = gtk_clist_insert (clist, i, insert_tab);
+			gtk_clist_set_row_data (clist, i, g_strdup (value));
+		}
+	}
+}
+static void
+save_settings (GtkCList *clist)
+{
+	gint i;
+	gchar *text;
+
+	profile_clean_section (panelize_section, profile_name);
+	for (i = 0; i < GTK_CLIST (clist)->rows; i++) {
+		gtk_clist_get_text (GTK_CLIST (clist), i, 0, &text);
+		WritePrivateProfileString (panelize_section,
+					   text,
+					   (gchar *) gtk_clist_get_row_data (GTK_CLIST (clist), i),
+					   profile_name);
+	}
+	sync_profiles ();
+}
 void
 gnome_external_panelize (GtkWidget *widget, WPanel *panel)
 {
-	GtkWidget *ep_dlg;
+	ep_dlg_data *data;
 	GtkWidget *frame;
-	GtkWidget *clist;
 	GtkWidget *sw;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
-	GtkWidget *button;
-	GtkWidget *entry;
+	gint i;
+	gchar *row_data;
 
-	ep_dlg = gnome_dialog_new (_("Run Command"), GNOME_STOCK_BUTTON_OK, 
+	data = g_new0 (ep_dlg_data, 1);
+	data->setting_text = FALSE;
+	data->selected = -1;
+	data->ep_dlg = gnome_dialog_new (_("Run Command"), GNOME_STOCK_BUTTON_OK, 
 				   GNOME_STOCK_BUTTON_CANCEL, NULL);
 
 				/* Frame 1 */
 	frame = gtk_frame_new (_("Preset Commands"));
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ep_dlg)->vbox),
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (data->ep_dlg)->vbox),
 			    frame, FALSE, FALSE, 0);
-	clist = gtk_clist_new (1);
-	gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 1, TRUE);
+	data->clist = gtk_clist_new (1);
+	load_settings (GTK_CLIST (data->clist));
+	gtk_signal_connect (GTK_OBJECT (data->clist), "select_row", GTK_SIGNAL_FUNC (ep_select_callback), (gpointer) data);
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_clist_columns_autosize (GTK_CLIST (data->clist));
+	gtk_clist_set_auto_sort (GTK_CLIST (data->clist), TRUE);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
-	sw = gtk_scrolled_window_new (GTK_CLIST (clist)->hadjustment, GTK_CLIST (clist)->vadjustment);
+	sw = gtk_scrolled_window_new (GTK_CLIST (data->clist)->hadjustment, GTK_CLIST (data->clist)->vadjustment);
 	gtk_widget_set_usize (sw, 300, 100);
-	gtk_container_add (GTK_CONTAINER (sw), clist);
+	gtk_container_add (GTK_CONTAINER (sw), data->clist);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-	button = gtk_button_new_with_label (_("Add"));
-	gtk_widget_set_usize (button, 75, 25);
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	button = gtk_button_new_with_label (_("Remove"));
-	gtk_widget_set_usize (button, 75, 25);
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	data->add_button = gtk_button_new_with_label (_("Add"));
+	gtk_signal_connect (GTK_OBJECT (data->add_button), "clicked", GTK_SIGNAL_FUNC (ep_add_callback), (gpointer) data);
+	gtk_widget_set_usize (data->add_button, 75, 25);
+	gtk_box_pack_end (GTK_BOX (hbox), data->add_button, FALSE, FALSE, 0);
+	data->remove_button = gtk_button_new_with_label (_("Remove"));
+	gtk_widget_set_sensitive (data->remove_button, FALSE);
+	gtk_signal_connect (GTK_OBJECT (data->remove_button), "clicked", GTK_SIGNAL_FUNC (ep_remove_callback), (gpointer) data);
+	gtk_widget_set_usize (data->remove_button, 75, 25);
+	gtk_box_pack_end (GTK_BOX (hbox), data->remove_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 				/* Frame 2 */
 	frame = gtk_frame_new (_("Run this Command"));
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (ep_dlg)->vbox),
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (data->ep_dlg)->vbox),
 			    frame, FALSE, FALSE, 0);
-	entry = gtk_entry_new ();
+	data->entry = gtk_entry_new ();
+	gtk_signal_connect (GTK_OBJECT (data->entry), "changed", GTK_SIGNAL_FUNC (ep_text_changed_callback), (gpointer) data);
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Command: ")), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), data->entry, TRUE, TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (frame), hbox);
-	gtk_widget_show_all (GNOME_DIALOG (ep_dlg)->vbox);
-	gnome_dialog_run_and_close (GNOME_DIALOG (ep_dlg));
+	gtk_widget_show_all (GNOME_DIALOG (data->ep_dlg)->vbox);
+	switch (gnome_dialog_run (GNOME_DIALOG (data->ep_dlg))) {
+	case 0:
+		gtk_widget_hide (data->ep_dlg);
+		while (gtk_events_pending () )
+			gtk_main_iteration ();
 
+		do_external_panelize (gtk_entry_get_text (GTK_ENTRY (data->entry)));
+		save_settings (GTK_CLIST (data->clist));
+		break;
+	case 1:
+	default:
+	}
+	for (i = 0; i < GTK_CLIST (data->clist)->rows; i++) {
+		row_data = gtk_clist_get_row_data (GTK_CLIST (data->clist), i);
+		if (row_data)
+			g_free (row_data);
+	}
+	gtk_widget_destroy (GTK_WIDGET (data->ep_dlg));
+	g_free (data);
 }
 void
 gnome_select_all_cmd (GtkWidget *widget, WPanel *panel)
