@@ -35,7 +35,7 @@
 #include "dlg.h"		/* draw_box, yes I know, it's silly */
 #include "background.h"		/* we_are_background definition */
 
-/* "$Id: dialog.c,v 1.3 1998/12/02 05:18:39 unammx Exp $" */
+/* "$Id: dialog.c,v 1.4 1999/01/01 02:05:05 unammx Exp $" */
 
 Refresh *refresh_list = 0;
 
@@ -79,6 +79,8 @@ void do_refresh (void)
 {
     if (we_are_background)
 	return;
+
+#ifndef HAVE_GNOME
     if (!refresh_list)
 	return;
     else {
@@ -88,6 +90,7 @@ void do_refresh (void)
 	    do_complete_refresh (refresh_list);
 	}
     }
+#endif
 }
 
 
