@@ -16,9 +16,9 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
+#define DIR_H_INCLUDE_HANDLE_DIRENT
 #include "tty.h"
 #include "fs.h"
-
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,13 +27,13 @@
 #include "x.h"
 #include "mad.h"
 #include "global.h"
-#define DIR_H_INCLUDE_HANDLE_DIRENT
 #include "dir.h"
 #include "util.h"
+#include "dialog.h"
 #include "tree.h"
 #include "../vfs/vfs.h"
 
-/* "$Id: dir.c,v 1.4 1998/05/26 00:53:36 unammx Exp $" */
+/* "$Id: dir.c,v 1.9 1998/12/24 21:37:38 norbert Exp $" */
 
 /* If true show files starting with a dot */
 int show_dot_files = 1;
@@ -295,10 +295,6 @@ void clean_dir (dir_list *list, int count)
     for (i = 0; i < count; i++){
 	free (list->list [i].fname);
 	list->list [i].fname = 0;
-	if (list->list [i].cache != NULL) {
-	    free (list->list [i].cache);
-	    list->list [i].cache = NULL;
-	}
     }
 }
 
@@ -320,9 +316,9 @@ add_dotdot_to_list (dir_list *list, int index)
 
     (list->list) [index].fnamelen = 2;
     (list->list) [index].fname = strdup ("..");
-    (list->list) [index].cache = NULL;
     (list->list) [index].f.link_to_dir = 0;
     (list->list) [index].f.stalled_link = 0;
+    (list->list) [index].f.dir_size_computed = 0;
     
     /* FIXME: We need to get the panel definition! to use file_mark */
     (list->list) [index].f.marked = 0;
@@ -378,8 +374,10 @@ int handle_dirent (dir_list *list, char *filter, struct dirent *dp,
     }
     if (!show_backups && dp->d_name [NLENGTH (dp)-1] == '~')
 	return 0;
-    if (mc_lstat (dp->d_name, buf1) == -1)
+    if (mc_lstat (dp->d_name, buf1) == -1) {
+	message(1, _(" Error "), _("File exists but can not be stat-ed: %s %s"), dp->d_name, strerror(errno));
         return 0;
+    }
 
     if (S_ISDIR (buf1->st_mode))
 	tree_check (dp->d_name);
@@ -472,10 +470,10 @@ int do_load_dir(dir_list *list, sortfn *sort, int reverse, int case_sensitive, c
 	    return next_free;
 	list->list [next_free].fnamelen = NLENGTH (dp);
 	list->list [next_free].fname = strdup (dp->d_name);
-	list->list [next_free].cache = NULL; 
 	list->list [next_free].f.marked = 0;
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stalled_link = stalled_link;
+        list->list [next_free].f.dir_size_computed = 0;
 	list->list [next_free].buf = buf;
 	if (strcmp (dp->d_name, ".." ) == 0)
 	    dotdot_found = 1;
@@ -499,14 +497,10 @@ int do_load_dir(dir_list *list, sortfn *sort, int reverse, int case_sensitive, c
 
 int link_isdir (file_entry *file)
 {
-    struct stat b;
-    
-    if (S_ISLNK (file->buf.st_mode)){
-	mc_stat (file->fname, &b);
-	if (S_ISDIR (b.st_mode))
-	    return 1;
-    }
-    return 0;
+    if (file->f.link_to_dir)
+	return 1;
+    else
+	return 0;
 }
  
 int if_link_is_exe (file_entry *file)
@@ -532,18 +526,15 @@ static void alloc_dir_copy (int size)
 	    for (i = 0; i < dir_copy.size; i++) {
 		if (dir_copy.list [i].fname)
 		    free (dir_copy.list [i].fname);
-		if (dir_copy.list [i].cache)
-		    free (dir_copy.list [i].cache);
 	    }
 	    free (dir_copy.list);
 	    dir_copy.list = 0;
 	}
 
 	dir_copy.list = xmalloc (sizeof (file_entry) * size, "alloc_dir_copy");
-	for (i = 0; i < size; i++) {
+	for (i = 0; i < size; i++)
 	    dir_copy.list [i].fname = 0;
-	    dir_copy.list [i].cache = NULL;
-	}
+
 	dir_copy.size = size;
     }
 }
@@ -571,8 +562,8 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
     for (i = 0; i < count; i++){
 	dir_copy.list [i].fnamelen = list->list [i].fnamelen;
 	dir_copy.list [i].fname =    list->list [i].fname;
-	dir_copy.list [i].cache =    list->list [i].cache;
 	dir_copy.list [i].f.marked = list->list [i].f.marked;
+        dir_copy.list [i].f.dir_size_computed = list->list [i].f.dir_size_computed;
 	dir_copy.list [i].f.link_to_dir = list->list [i].f.link_to_dir;
 	dir_copy.list [i].f.stalled_link = list->list [i].f.stalled_link;
     }
@@ -602,7 +593,7 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
 	    if (tmp_len == dir_copy.list [i].fnamelen
 		&& !strcmp (dp->d_name, dir_copy.list [i].fname)){
 		list->list [next_free].f.marked = dir_copy.list [i].f.marked;
-		found = 1;
+                found = 1;
 		break;
 	    }
 	
@@ -611,9 +602,9 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
 	
 	list->list [next_free].fnamelen = tmp_len;
 	list->list [next_free].fname = strdup (dp->d_name);
-	list->list [next_free].cache = NULL;
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stalled_link = stalled_link;
+        list->list [next_free].f.dir_size_computed = 0;
 	list->list [next_free].buf = buf;
 	if (strcmp (dp->d_name, ".." ) == 0)
 	    dotdot_found = 1;
