@@ -17,7 +17,7 @@
  */
 
 #include <config.h>
-/* "$Id: dlg.c,v 1.19 1998/09/25 18:51:03 unammx Exp $" */
+/* "$Id: dlg.c,v 1.23 1998/12/02 23:44:22 unammx Exp $" */
 #include <string.h>
 #include <stdio.h>
 #include <malloc.h>
@@ -59,6 +59,15 @@ Hook *idle_hook = 0;
 #   define x_dialog_stop(d)
 #endif
 
+#ifdef HAVE_X
+void widget_erase (Widget *w)
+{
+}
+
+void dlg_erase (Dlg_head *h)
+{
+}
+#else
 static void slow_box (Dlg_head *h, int y, int x, int ys, int xs)
 {
     move (h->y+y, h->x+x);
@@ -130,6 +139,7 @@ void dlg_erase (Dlg_head *h)
 	}
     }
 }
+#endif /* HAVE_X */
 
 void init_widget (Widget *w, int y, int x, int lines, int cols,
 		  int (*callback)(Dlg_head *, void *, int, int),
@@ -216,10 +226,13 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
 {
     Dlg_head *new_d;
 
+#ifndef HAVE_X
     if (flags & DLG_CENTER){
 	y1 = (LINES-lines)/2;
 	x1 = (COLS-cols)/2;
     }
+#endif
+    
     if ((flags & DLG_TRYUP) && (y1 > 3))
 	y1 -= 2;
 
@@ -607,7 +620,8 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
     case XCTRL('z'):
 	suspend_cmd ();
 	/* Fall through */
-	    
+
+#ifndef HAVE_X
     case XCTRL('l'):
 #ifndef HAVE_SLANG
 	/* Use this if the refreshes fail */
@@ -618,6 +632,7 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 #endif
 	mc_refresh ();
 	doupdate ();
+#endif
 	break;
 	
     case '\n':
@@ -642,7 +657,6 @@ static int dlg_try_hotkey (Dlg_head *h, int d_key)
     Widget_Item *hot_cur;
     Widget_Item *previous;
     int    handled, c;
-    extern input_event ();
     
     /*
      * Explanation: we don't send letter hotkeys to other widgets if
@@ -733,6 +747,7 @@ int dlg_key_event (Dlg_head *h, int d_key)
 
 	return handled;
     }
+    return 1;
 }
 
 static INLINE int dlg_mouse_event (Dlg_head *h, Gpm_Event *event)
@@ -778,16 +793,19 @@ void init_dlg (Dlg_head *h)
 {
     int refresh_mode;
 
-    tk_end_frame ();
-    
     /* Initialize dialog manager and widgets */
     (*h->callback) (h, 0, DLG_INIT);
     dlg_broadcast_msg (h, WIDGET_INIT, 0);
 
+#ifdef HAVE_X
+    refresh_mode = REFRESH_COVERS_PART;
+#else
     if (h->x == 0 && h->y == 0 && h->cols == COLS && h->lines == LINES)
 	refresh_mode = REFRESH_COVERS_ALL;
     else
 	refresh_mode = REFRESH_COVERS_PART;
+    
+#endif
     push_refresh (dlg_refresh, h, refresh_mode);
     h->refresh_pushed = 1;
     
@@ -910,8 +928,11 @@ destroy_dlg (Dlg_head *h)
 	free (h->title);
     x_destroy_dlg (h);
     free (h);
+
+#ifndef HAVE_X
     if (refresh_list)
 	do_refresh ();
+#endif
 }
 
 int std_callback (Dlg_head *h, int Msg, int Par)
@@ -999,54 +1020,6 @@ int dlg_select_nth_widget (Dlg_head *h, int n)
     return dlg_select_widget (h, w->widget);
 }
 
-#ifdef HAVE_TK
-/* Frames must include a trailing dot */
-static void tk_frame_proc (Dlg_head *h, char *frame, int new_frame)
-{
-    char *s = strdup (frame);
-    
-    if (frame [strlen (frame)-1] != '.'){
-	fprintf (stderr, "Invalid frame name\n");
-	exit (1);
-    }
-    s [strlen (frame)-1] = 0;
-    the_frame = frame;
-    
-    if (new_frame)
-	tk_evalf ("frame %s.%s", (char *)h->wdata, s);
-}
-
-/* If passed a null string, it returns */
-void tk_new_frame (Dlg_head *h, char *frame)
-{
-    if (!*frame)
-	return;
-    tk_frame_proc (h, frame, 1);
-}
-
-void tk_frame (Dlg_head *h, char *frame)
-{
-    tk_frame_proc (h, frame, 0);
-}
-
-void tk_end_frame ()
-{
-    the_frame = "";
-}
-#else
-void tk_new_frame (Dlg_head *h, char *x)
-{
-}
-
-void tk_frame (Dlg_head *h, char *x)
-{
-}
-
-void tk_end_frame (void)
-{
-}
-#endif
-
 #ifndef PORT_HAS_DIALOG_TITLE
 void
 x_set_dialog_title (Dlg_head *h, char *title)
