@@ -29,11 +29,8 @@
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
-#include <sys/types.h>		/* FD_ZERO et al */
-#ifndef SCO_FLAVOR
-	/* alex: sys/select.h defines struct timeval */
-#	include <sys/time.h>		/* struct timeval */
-#endif /* SCO_FLAVOR */
+#include <sys/types.h>
+#include <sys/time.h>
 #if HAVE_SYS_SELECT_H
 #   include <sys/select.h>
 #endif
@@ -47,7 +44,6 @@
 #include "mouse.h"
 #include "key.h"
 #include "main.h"
-#include "file.h"
 #include "win.h"
 #include "cons.saver.h"
 #include "../vfs/vfs.h"
@@ -63,10 +59,7 @@
 
 #include "x.h"
 
-/* "$Id: key.c,v 1.7 1998/05/31 20:28:12 norbert Exp $" */
-
-/* This macros were stolen from gpm 0.15 */
-#define GET_TIME(tv) (gettimeofday(&tv, (struct timezone *)NULL))
+#define GET_TIME(tv)    (gettimeofday(&tv, (struct timezone *)NULL))
 #define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
 			 (t2.tv_usec-t1.tv_usec)/1000)
 			 
@@ -226,7 +219,8 @@ key_define_t mc_default_keys [] = {
 };
 #endif
 
-void define_sequences (key_define_t *kd)
+static void
+define_sequences (key_define_t *kd)
 {
 #ifndef HAVE_X
     int i;
@@ -289,7 +283,8 @@ void init_key_input_fd (void)
 
 
 #ifndef HAVE_X
-void xmouse_get_event (Gpm_Event *ev)
+static void
+xmouse_get_event (Gpm_Event *ev)
 {
     int btn;
     static struct timeval tv1 = { 0, 0 }; /* Force first click as single */
@@ -419,7 +414,8 @@ void define_sequence (int code, char *seq, int action)
 static int *pending_keys;
 #endif
 
-int correct_key_code (int c)
+static int
+correct_key_code (int c)
 {
     /* This is needed on some OS that do not support ncurses and */
     /* do some magic after read()ing the data */
@@ -618,7 +614,8 @@ int get_key_code (int no_delay)
 
 #ifndef PORT_HAS_FILE_HANDLERS
 /* If set timeout is set, then we wait 0.1 seconds, else, we block */
-void try_channels (int set_timeout)
+static void
+try_channels (int set_timeout)
 {
     struct timeval timeout;
     static fd_set select_set;
@@ -677,13 +674,13 @@ static int getch_with_delay (void)
 
 extern int max_dirt_limit;
 
+#ifndef HAVE_X
 /* Returns a character read from stdin with appropriate interpretation */
 /* Also takes care of generated mouse events */
 /* Returns EV_MOUSE if it is a mouse event */
 /* Returns EV_NONE  if non-blocking or interrupt set and nothing was done */
 int get_event (Gpm_Event *event, int redo_event, int block)
 {
-#ifndef HAVE_X
     int c;
     static int flag;			/* Return value from select */
 #ifdef HAVE_LIBGPM
@@ -792,9 +789,8 @@ int get_event (Gpm_Event *event, int redo_event, int block)
     flag = is_wintouched(stdscr);
     untouchwin (stdscr);
 #   endif
-
     c = block ? getch_with_delay () : get_key_code(1);
-    
+
 #   ifndef HAVE_SLANG
     if (flag)
         touchwin (stdscr);
@@ -806,10 +802,14 @@ int get_event (Gpm_Event *event, int redo_event, int block)
     }
 
     return c;
-#else
-    return EV_NONE;
-#endif /* HAVE_X */
 }
+#else
+/* X11 version of the get_event routine */
+int get_event (Gpm_Event *event, int redo_event, int block)
+{
+    return EV_NONE;
+}
+#endif
 
 #ifndef PORT_HAS_GETCH
 /* Returns a key press, mouse events are discarded */
@@ -823,7 +823,6 @@ int mi_getch ()
 	;
     return key;
 }
-#endif
 
 int xgetch_second (void)
 {
@@ -849,9 +848,11 @@ int xgetch_second (void)
 #endif
     return c;
 }
+#endif
 
 #ifndef HAVE_X
-void learn_store_key (char *buffer, char **p, int c)
+static void
+learn_store_key (char *buffer, char **p, int c)
 {
     if (*p - buffer > 253)
         return;
@@ -963,7 +964,8 @@ int is_idle (void)
 }
 
 
-int get_modifier ()
+int
+get_modifier (void)
 {
 #ifdef __linux__
     unsigned char modifiers;
@@ -979,7 +981,8 @@ int get_modifier ()
 #endif
 }
 
-int ctrl_pressed ()
+int
+ctrl_pressed ()
 {
 #ifdef __linux__
     if (get_modifier () & CONTROL_PRESSED)
