@@ -57,6 +57,7 @@
 #include "main.h"
 #include "global.h"
 #include "../vfs/vfs.h"
+#include "panelize.h"
 
 void do_external_panelize (char *command);
 
@@ -254,7 +255,8 @@ static void add2panelize (char *label, char *command)
     }
 }
 
-void add2panelize_cmd (void)
+void
+add2panelize_cmd (void)
 {
     char *label;
 
@@ -293,7 +295,8 @@ static void remove_from_panelize (struct panelize *entry)
     }
 }
 
-void external_panelize (void)
+void
+external_panelize (void)
 {
     char *target = NULL;
 
@@ -408,7 +411,9 @@ void do_external_panelize (char *command)
 
     /* Clear the counters */
     cpanel->total = cpanel->dirs_marked = cpanel->marked = 0;
-    cpanel->has_dir_sizes = 0;
+    cpanel->top_file = 0;
+    cpanel->selected = 0;
+
     while (1) {
 	clearerr(external);
 	if (fgets (line, MC_MAXPATHLEN, external) == NULL) {
@@ -433,18 +438,19 @@ void do_external_panelize (char *command)
 	    break;
 	list->list [next_free].fnamelen = strlen (name);
 	list->list [next_free].fname = strdup (name);
-	list->list [next_free].cache = NULL;
 	file_mark (cpanel, next_free, 0);
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stalled_link = stalled_link;
+	list->list [next_free].f.dir_size_computed = 0;
 	list->list [next_free].buf = buf;
 	next_free++;
 	if (!(next_free & 32))
 	    rotate_dash ();
     }
+
+    cpanel->is_panelized = 1;
     if (next_free){
 	cpanel->count = next_free;
-	cpanel->is_panelized = 1;
 	if (list->list [0].fname [0] == PATH_SEP){
 	    strcpy (cpanel->cwd, PATH_SEP_STR);
 	    chdir (PATH_SEP_STR);
