@@ -68,17 +68,11 @@ extern int command_prompt;
 /* Specifies the display mode: 1d or 2d */
 int tree_navigation_flag;
 
-/* If this is true, then when browsing the tree the other window will
- * automatically reload it's directory with the contents of the currently
- * selected directory.
- */
-int xtree_mode = 0;
-
 /* Forwards */
 static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par);
 #define tcallback (callback_fn) tree_callback
 
-/* "$Id: tree.c,v 1.9 1998/09/22 16:13:54 proskin Exp $" */
+/* "$Id: tree.c,v 1.13 1998/12/30 19:19:24 unammx Exp $" */
 
 /* Returns number of common characters */
 static inline int str_common (char *s1, char *s2)
@@ -135,7 +129,7 @@ static tree_entry *forw_ptr (tree_entry *ptr, int *count)
     greater than zero if p1 is found to be less than, to match, or be greater 
     than p2.
  */
-int
+static int
 pathcmp (const char *p1, const char *p2)
 {
     for ( ;*p1 == *p2; p1++, p2++)
@@ -950,6 +944,12 @@ void tree_chdir (WTree *tree, char *dir)
     }
 }
 
+void
+sync_tree (char *path)
+{
+    tree_chdir (the_tree, path);
+}
+
 /* Handle mouse click */
 void tree_event (WTree *tree, int y)
 {
@@ -1093,8 +1093,10 @@ static int toggle_nav_mode (void)
 
 void tree_copy (WTree *tree, char *default_dest)
 {
-    char *dest;
-
+    char   *dest;
+    long   count = 0;
+    double bytes = 0;
+    
     if (!tree->selected_ptr)
 	return;
     sprintf (cmd_buf, _("Copy \"%s\" directory to:"),
@@ -1105,7 +1107,8 @@ void tree_copy (WTree *tree, char *default_dest)
     }
     create_op_win (OP_COPY, 0);
     file_mask_defaults ();
-    copy_dir_dir (tree->selected_ptr->name, dest, 1, 0, 0, 0);
+
+    copy_dir_dir (tree->selected_ptr->name, dest, 1, 0, 0, 0, &count, &bytes);
     destroy_op_win ();
     free (dest);
 }
@@ -1125,9 +1128,11 @@ static int tree_copy_cmd (WTree *tree)
 
 void tree_move (WTree *tree, char *default_dest)
 {
-    char *dest;
+    char   *dest;
     struct stat buf;
-
+    double bytes = 0;
+    long   count = 0;
+    
     if (!tree->selected_ptr)
 	return;
     sprintf (cmd_buf, _("Move \"%s\" directory to:"),
@@ -1149,18 +1154,20 @@ void tree_move (WTree *tree, char *default_dest)
     }
     create_op_win (OP_MOVE, 0);
     file_mask_defaults ();
-    move_dir_dir (tree->selected_ptr->name, dest);
+    move_dir_dir (tree->selected_ptr->name, dest, &count, &bytes);
     destroy_op_win ();
     free (dest);
 }
 
-static int tree_move_cmd (WTree *tree)
+static int
+tree_move_cmd (WTree *tree)
 {
     tree_move (tree, "");
     return 1;
 }
 
-static int tree_mkdir_cmd (WTree *tree)
+static int
+tree_mkdir_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
@@ -1178,10 +1185,13 @@ static int tree_mkdir_cmd (WTree *tree)
     return 1;
 }
 
-static void tree_rmdir_cmd (WTree *tree)
+static void
+tree_rmdir_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
-
+    long count = 0;
+    double bytes = 0;
+    
     if (tree->selected_ptr){
 	if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
 	    return;
@@ -1201,7 +1211,7 @@ static void tree_rmdir_cmd (WTree *tree)
 	    }
 	}
 	create_op_win (OP_DELETE, 0);
-	if (erase_dir (tree->selected_ptr->name) == FILE_CONT)
+	if (erase_dir (tree->selected_ptr->name, &count, &bytes) == FILE_CONT)
 	    tree_forget_cmd (tree);
 	destroy_op_win ();
 	mc_chdir (old_dir);
@@ -1211,7 +1221,8 @@ static void tree_rmdir_cmd (WTree *tree)
 }
 
 #if 0
-static int tree_quit_cmd (void)
+static int
+tree_quit_cmd (void)
 {
     /*
        FIXME
@@ -1222,48 +1233,56 @@ static int tree_quit_cmd (void)
 #endif
 
 static void set_navig_label (Dlg_head *h);
-static void tree_toggle_navig (Dlg_head *h)
+
+static void
+tree_toggle_navig (Dlg_head *h)
 {
     tree_navigation_flag = 1 - tree_navigation_flag;
     set_navig_label (h);
 }
 
-void set_navig_label (Dlg_head *h)
+void
+set_navig_label (Dlg_head *h)
 {
     define_label_data (h, (Widget *)tree,
 		       4, tree_navigation_flag ? _("Static") : _("Dynamc"),
 		       (void (*)(void *))tree_toggle_navig, h);
 }
 
-static void move_down (WTree *tree)
+static void
+move_down (WTree *tree)
 {
     tree_move_forward (tree, 1);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static void move_up (WTree *tree)
+static void
+move_up (WTree *tree)
 {
     tree_move_backward (tree, 1);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static void move_home (WTree *tree)
+static void
+move_home (WTree *tree)
 {
     tree_move_to_top (tree);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static void move_end (WTree *tree)
+static void
+move_end (WTree *tree)
 {
     tree_move_to_bottom (tree);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static int move_left (WTree *tree)
+static int
+move_left (WTree *tree)
 {
     int v;
     
@@ -1276,7 +1295,8 @@ static int move_left (WTree *tree)
     return 0;
 }
 
-static int move_right (WTree *tree)
+static int
+move_right (WTree *tree)
 {
     if (tree_navigation_flag){
 	tree_move_to_child (tree);
@@ -1287,21 +1307,24 @@ static int move_right (WTree *tree)
     return 0;
 }
 
-static void move_prevp (WTree *tree)
+static void
+move_prevp (WTree *tree)
 {
     tree_move_backward (tree, tlines (tree) - 1);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static void move_nextp (WTree *tree)
+static void
+move_nextp (WTree *tree)
 {
     tree_move_forward (tree, tlines (tree) - 1);
     show_tree (tree);
     maybe_chdir (tree);
 }
 
-static void chdir_sel (WTree *tree)
+static void
+chdir_sel (WTree *tree)
 {
     if (!tree->is_panel){
 	tree->done = 1;
@@ -1320,7 +1343,8 @@ static void chdir_sel (WTree *tree)
     return;
 }
 
-static void start_search (WTree *tree)
+static void
+start_search (WTree *tree)
 {
     int i;
 
@@ -1371,7 +1395,8 @@ static key_map tree_keymap [] = {
     { 0, 0 }
     };
 
-static inline int tree_key (WTree *tree, int key)
+static inline int
+tree_key (WTree *tree, int key)
 {
     int i;
 
@@ -1422,7 +1447,8 @@ static inline int tree_key (WTree *tree, int key)
     return 0;
 }
 
-static void tree_frame (Dlg_head *h, WTree *tree)
+static void
+tree_frame (Dlg_head *h, WTree *tree)
 {
     attrset (NORMAL_COLOR);
     widget_erase ((Widget*) tree);
@@ -1437,7 +1463,8 @@ static void tree_frame (Dlg_head *h, WTree *tree)
 }
 
 
-static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
+static int
+tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
 {
     switch (msg){
     case WIDGET_DRAW:
@@ -1487,7 +1514,8 @@ static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
     return default_proc (h, msg, par);
 }
 
-WTree *tree_new (int is_panel, int y, int x, int lines, int cols)
+WTree *
+tree_new (int is_panel, int y, int x, int lines, int cols)
 {
     WTree *tree = xmalloc (sizeof (WTree), "tree_new");
 
@@ -1509,87 +1537,5 @@ WTree *tree_new (int is_panel, int y, int x, int lines, int cols)
     return tree;
 }
 
-static char *get_absolute_name (char *file)
-{
-    char dir [MC_MAXPATHLEN];
-
-    if (file [0] == PATH_SEP)
-	return strdup (file);
-    mc_get_current_wd (dir, MC_MAXPATHLEN);
-    return get_full_name (dir, file);
-}
-
-static int my_mkdir_rec (char *s, mode_t mode)
-{
-    char *p, *q;
-    int result;
-    
-    if (!mc_mkdir (s, mode))
-        return 0;
-
-    /* FIXME: should check instead if s is at the root of that filesystem */
-    if (!vfs_file_is_local (s))
-	return -1;
-    if (!strcmp (s, PATH_SEP_STR))
-        return ENOTDIR;
-    p = concat_dir_and_file (s, "..");
-    q = vfs_canon (p);
-    free (p);
-    if (!(result = my_mkdir_rec (q, mode))) {
-    	result = mc_mkdir (s, mode);
-    } 
-    free (q);
-    return result;
-}
-
-int my_mkdir (char *s, mode_t mode)
-{
-    int result;
-#if FIXME    
-    WTree *tree = 0;
-#endif    
-
-    result = mc_mkdir (s, mode);
-#ifdef OS2_NT
-    /* .ado: it will be disabled in OS/2 and NT */
-    /* otherwise crash if directory already exists. */
-    return result;
-#endif
-    if (result) {
-        char *p = vfs_canon (s);
-        
-        result = my_mkdir_rec (p, mode);
-        free (p);
-    }
-    if (result == 0){
-	s = get_absolute_name (s);
-#if FIXME
-	/* FIXME: Should receive a Wtree! */
-
-	tree_add_entry (tree, s);
-#endif
-	free (s);
-    }
-    return result;
-}
-
-int my_rmdir (char *s)
-{
-    int result;
-#if FIXME    
-    WTree *tree = 0;
-#endif    
-
-    /* FIXME: Should receive a Wtree! */
-    result = mc_rmdir (s);
-    if (result == 0){
-	s = get_absolute_name (s);
-#if FIXME
-	tree_remove_entry (tree, s);
-#endif
-	free (s);
-    }
-    return result;
-}
 
 
