@@ -3,6 +3,10 @@
 int got_interrupt (void);
 void mc_refresh (void);
 
+#ifdef HAVE_X
+#include "xtty.h"
+#else /* HAVE_X */
+
 #ifdef HAVE_SLANG
 #   include "myslang.h"
 
@@ -87,7 +91,7 @@ void mc_refresh (void);
 
 #   define ACS_MAP(x) '*'
 
-#   define NO_COLOR_SUPPORT
+#   define NO_COLOR_CURSES
 #   define untouchwin(win) 
 #   define xgetch x_getch
 #   define wtouchln(win,b,c,d) touchwin(win)
@@ -123,7 +127,7 @@ void mc_refresh (void);
 
 #endif /* USE_SYSV_CURSES */
 
-#ifdef NO_COLOR_SUPPORT
+#ifdef NO_COLOR_CURSES
 #   define COLOR_PAIR(x) 1
 
 enum {
@@ -139,3 +143,4 @@ int init_pair (int, int, int);
 #define KEY_KP_SUBTRACT	4002
 #define KEY_KP_MULTIPLY	4003
 
+#endif /* HAVE_X */
