@@ -28,7 +28,7 @@
 
 /* }}} */
 
-/*  [] = "$Id: wtools.c,v 1.10 1998/05/19 18:37:08 unammx Exp $" */
+/*  [] = "$Id: wtools.c,v 1.13 1998/12/02 23:44:33 unammx Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -47,7 +47,7 @@
 #include "menu.h"
 #include "wtools.h"
 #include "key.h"	/* For mi_getch() */
-#include "dialog.h"	/* For do_refresh() and my_wputs() */
+#include "dialog.h"	/* For do_refresh() */
 #include "complete.h"   /* INPUT_COMPLETE_CD */
 #include "x.h"
 
@@ -89,7 +89,7 @@ common_dialog_callback (struct Dlg_head *h, int id, int msg)
 /* }}} */
 /* {{{ Listbox utility functions */
 
-void
+static void
 listbox_refresh (Dlg_head *h)
 {
     dialog_repaint (h, COLOR_NORMAL, COLOR_HOT_NORMAL);
@@ -265,25 +265,15 @@ int query_dialog (char *header, char *text, int flags, int count, ...)
 	    xpos = strlen (cur_name)+6;
 		if (strchr(cur_name, '&') != NULL)
 			xpos--;
-#ifndef HAVE_XVIEW
 	    add_widget (query_dlg, button_new
 			(lines-3, cols, B_USER+i, NORMAL_BUTTON, cur_name,
 			 0, 0, NULL));
-#else			 
-	    buttonnames [i] = cur_name;
-#endif			     
 	    cols += xpos;
 	    if (i == sel_pos)
 		query_dlg->initfocus = query_dlg->current;
 	}
 	va_end (ap);
 
-#ifdef HAVE_XVIEW
-	for (i = count - 1; i >= 0; i--)
-	    add_widgetl (query_dlg, button_new
-			 (0, 0, B_USER+i, NORMAL_BUTTON, buttonnames [i], 0, 0, NULL), 
-			 i ? XV_WLAY_RIGHTOF : XV_WLAY_CENTERROW);
-#endif	
 	add_widget (query_dlg, label_new (2, 3, text, NULL));
 	
 	/* run dialog and make result */
@@ -345,15 +335,8 @@ Dlg_head *message (int error, char *header, char *text, ...)
     va_end (args);
     
     query_dialog (header, buffer, error, 0);
-#ifndef HAVE_XVIEW
+
     d = last_query_dlg;
-#ifdef HAVE_TK
-    if (error & D_INSERT){
-	init_dlg (d);
-	tk_dispatch_all ();
-	return d;
-    }
-#else	
     init_dlg (d);
     if (!(error & D_INSERT)){
 	mi_getch ();
@@ -361,8 +344,6 @@ Dlg_head *message (int error, char *header, char *text, ...)
 	destroy_dlg (d);
     } else
 	return d;
-#endif
-#endif	
     return 0;
 }
 #endif
@@ -654,7 +635,8 @@ char *input_dialog (char *header, char *text, char *def_text)
     return input_dialog_help (header, text, "[Input Line Keys]", def_text);
 }
 
-int input_dialog_help_2 (char *header, char *text1, char *text2, char *help, char **r1, char **r2)
+static int
+input_dialog_help_2 (char *header, char *text1, char *text2, char *help, char **r1, char **r2)
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
