@@ -23,7 +23,15 @@
  * strings you pass to them. This is acutally ok as you strdup what
  * you are passing to them, anyway; still, beware.  */
 
+/* Namespace: exports *many* functions with vfs_ prefix; exports
+   parse_ls_lga and friends which do not have that prefix. */
+
 #include <config.h>
+#include <glib.h>
+#undef MIN
+#undef MAX
+
+#include <syslog.h>
 #include <stdio.h>
 #include <stdlib.h>	/* For atol() */
 #include <stdarg.h>
@@ -50,9 +58,8 @@
 #include "../src/dialog.h"
 #endif
 #include "vfs.h"
-#include "mcfs.h"
+#include "extfs.h"		/* FIXME: we should not know anything about our modules */
 #include "names.h"
-#include "extfs.h"
 #ifdef USE_NETCODE
 #   include "tcputil.h"
 #endif
@@ -65,8 +72,8 @@ int vfs_flags = 0;    /* Flags */
 extern int cd_symlinks; /* Defined in main.c */
 
 /* They keep track of the current directory */
-static vfs *current_vfs = &local_vfs_ops;
-char *current_dir = NULL;
+static vfs *current_vfs = &vfs_local_ops;
+static char *current_dir = NULL;
 
 /*
  * FIXME: this is broken. It depends on mc not crossing border on month!
@@ -78,14 +85,16 @@ static int current_year;
 uid_t vfs_uid = 0;
 gid_t vfs_gid = 0;
 
-/* Open files managed by the vfs layer */
+/* FIXME: Open files managed by the vfs layer, should be dynamical */
 #define MAX_VFS_FILES 100
+
 static struct {
     void *fs_info;
     vfs  *operations;
 } vfs_file_table [MAX_VFS_FILES];
 
-static int get_bucket (void)
+static int
+get_bucket (void)
 {
     int i;
 
@@ -94,48 +103,83 @@ static int get_bucket (void)
 	if (!vfs_file_table [i].fs_info)
 	    return i;
     }
+
     vfs_die ("No more virtual file handles");
-    return 0; /* Shut up, stupid gcc */
+    return 0;
 }
 
-static vfs *vfs_list = &local_vfs_ops;	/* It _has_ to be the first */
+/* vfs_local_ops needs to be the first one */
+static vfs *vfs_list = &vfs_local_ops;
 
-#define FOR_ALL_VFS for (vfs=vfs_list; vfs; vfs=vfs->next) 
-
-int vfs_register( vfs *vfs )
+int
+vfs_register (vfs *vfs)
 {
-    int res = (vfs->init) ? (*vfs->init)(vfs) : 1;
+    int res;
+
+    if (!vfs) vfs_die("You can not register NULL.");
+    res = (vfs->init) ? (*vfs->init)(vfs) : 1;
+
     if (!res) return 0;
+
     vfs->next = vfs_list;
     vfs_list = vfs;
-#if 0
-    fprintf( stderr, "VFS %s successfully registered\n", vfs->name );
-#endif
+
     return 1;
 }
 
-vfs *vfs_type_from_op (char *path)
+vfs *
+vfs_type_from_op (char *path)
 {
     vfs *vfs;
-    FOR_ALL_VFS {
-        if (vfs == &local_vfs_ops)	/* local catches all */ 
-	    return NULL;
+
+    if (!path) vfs_die( "vfs_type_from_op got NULL: impossible" );
+    
+    for (vfs = vfs_list; vfs != &vfs_local_ops; vfs = vfs->next){
         if (vfs->which) {
-	    if ((*vfs->which) (vfs, path) != -1)
-	        return vfs;
-	    else
-	        continue;
+	    if ((*vfs->which) (vfs, path) == -1)
+		continue;
+	    return vfs;
 	}
 	if (!strncmp (path, vfs->prefix, strlen (vfs->prefix)))
 	    return vfs;
     }
-    vfs_die( "No local in vfs list?" );
     return NULL; /* shut up stupid gcc */
 }
 
-int path_magic( char *path )
+/* Strip known vfs suffixes from a filename (possible improvement: strip
+   suffix from last path component). 
+   Returns a malloced string which has to be freed. */
+char *
+vfs_strip_suffix_from_filename (char *filename)
+{
+    vfs *vfs;
+    char *semi;
+    char *p;
+
+    if (!filename) vfs_die( "vfs_strip_suffix_from_path got NULL: impossible" );
+    
+    p = strdup (filename);
+    if (!(semi = strrchr (p, '#')))
+	return p;
+
+    for (vfs = vfs_list; vfs != &vfs_local_ops; vfs = vfs->next){
+        if (vfs->which) {
+	    if ((*vfs->which) (vfs, semi + 1) == -1)
+		continue;
+	    *semi = '\0'; /* Found valid suffix */
+	    return p;
+	}
+	if (!strncmp (semi + 1, vfs->prefix, strlen (vfs->prefix))) {
+	    *semi = '\0'; /* Found valid suffix */
+	    return p;
+        }
+    }
+    return p;
+}
+
+static int
+path_magic (char *path)
 {
-    int res;
     struct stat buf;
 
     if (vfs_flags & FL_ALWAYS_MAGIC)
@@ -152,29 +196,39 @@ int path_magic( char *path )
  * What is left in path is p1. You still want to free(path), you DON'T
  * want to free neither *inpath nor *op
  */
-vfs *vfs_split (char *path, char **inpath, char **op)
+vfs *
+vfs_split (char *path, char **inpath, char **op)
 {
-    char *semi = strrchr (path, '#');
+    char *semi;
     char *slash;
     vfs *ret;
+
+    if (!path) vfs_die("Can not split NULL");
     
+    semi = strrchr (path, '#');
     if (!semi || !path_magic(path))
 	return NULL;
+
     slash = strchr (semi, '/');
     *semi = 0;
+
     if (op)
 	*op = NULL;
+
     if (inpath)
 	*inpath = NULL;
+
     if (slash)
 	*slash = 0;
+
     if ((ret = vfs_type_from_op (semi+1))){
 	if (op) 
-	    *op = semi+1;
+	    *op = semi + 1;
 	if (inpath)
-	    *inpath = slash?slash+1:NULL;
+	    *inpath = slash ? slash + 1 : NULL;
 	return ret;
     }
+
     if (slash)
 	*slash = '/';
     ret = vfs_split (path, inpath, op);
@@ -182,51 +236,65 @@ vfs *vfs_split (char *path, char **inpath, char **op)
     return ret;
 }
 
-vfs*
+vfs *
 vfs_rosplit (char *path)
 {
-    char *semi = strrchr (path, '#');
+    char *semi;
     char *slash;
     vfs *ret;
+
+    if (!path) vfs_die( "Can not rosplit NULL" );
+    semi = strrchr (path, '#');
     
-    if (!semi || !path_magic(path))
+    if (!semi || !path_magic (path))
 	return NULL;
+    
     slash = strchr (semi, '/');
     *semi = 0;
     if (slash)
 	*slash = 0;
     
     ret = vfs_type_from_op (semi+1);
+
     if (slash)
 	*slash = '/';
     if (!ret)
 	ret = vfs_rosplit (path);
+
     *semi = '#';
     return ret;
 }
 
-vfs *vfs_type (char *path)
+vfs *
+vfs_type (char *path)
 {
-    vfs *vfs = vfs_rosplit(path);
+    vfs *vfs;
+
+    vfs = vfs_rosplit(path);
+
     if (!vfs)
-        vfs = &local_vfs_ops;
+        vfs = &vfs_local_ops;
+
     return vfs;
 }
 
 static struct vfs_stamping *stamps;
 
-/* Returns the number of seconds remaining to the vfs timeout
+/*
+ * Returns the number of seconds remaining to the vfs timeout
  *
  * FIXME: currently this is set to 10 seconds.  We should compute this.
  */
-int vfs_timeouts ()
+int
+vfs_timeouts ()
 {
     return stamps ? 10 : 0;
 }
 
-void vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
+void
+vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
 {
-    if (v != &local_vfs_ops && id != (vfsid)-1){
+    if (v != &vfs_local_ops && id != (vfsid)-1){
         struct vfs_stamping *stamp, *st1;
         
         for (stamp = stamps; stamp != NULL; st1 = stamp, stamp = stamp->next)
@@ -249,8 +317,10 @@ void vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
 	}
 	else
     	    stamp->parent = 0;
+	
         gettimeofday (&(stamp->time), NULL);
         stamp->next = 0;
+
 	if (stamps)
 	    st1->next = stamp;
 	else
@@ -258,30 +328,37 @@ void vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
     }
 }
 
-void vfs_stamp (vfs *v, vfsid id)
+void
+vfs_stamp (vfs *v, vfsid id)
 {
     struct vfs_stamping *stamp;
-    
+
     for (stamp = stamps; stamp != NULL; stamp = stamp->next)
         if (stamp->v == v && stamp->id == id){
+
             gettimeofday (&(stamp->time), NULL);
             if (stamp->parent != NULL)
                 vfs_stamp (stamp->parent->v, stamp->parent->id);
+
             return;
         }
 }
 
-void vfs_rm_parents (struct vfs_stamping *stamp)
+void
+vfs_rm_parents (struct vfs_stamping *stamp)
 {
     struct vfs_stamping *st2, *st3;
+
     if (stamp){
 	for (st2 = stamp, st3 = st2->parent; st3 != NULL; st2 = st3, st3 = st3->parent)
 	    free (st2);
+
 	free (st2);
     }
 }
 
-void vfs_rmstamp (vfs *v, vfsid id, int removeparents)
+void
+vfs_rmstamp (vfs *v, vfsid id, int removeparents)
 {
     struct vfs_stamping *stamp, *st1;
     
@@ -298,6 +375,7 @@ void vfs_rmstamp (vfs *v, vfsid id, int removeparents)
             	st1->next = stamp->next;
             }
             free (stamp);
+
             return;
         }
 }
@@ -305,11 +383,12 @@ void vfs_rmstamp (vfs *v, vfsid id, int removeparents)
 static int
 ferrno (vfs *vfs)
 {
-    return vfs->ferrno ? (*vfs->ferrno)(vfs) : EOPNOTSUPP; 
+    return vfs->ferrno ? (*vfs->ferrno)(vfs) : E_UNKNOWN; 
     /* Hope that error message is obscure enough ;-) */
 }
 
-int mc_open (char *file, int flags, ...)
+int
+mc_open (char *file, int flags, ...)
 {
     int  handle;
     int  mode;
@@ -326,7 +405,8 @@ int mc_open (char *file, int flags, ...)
     va_end (ap);
     
     if (!vfs->open)
-        vfs_die( "VFS must support open.\n" );
+	vfs_die ("VFS must support open.\n");
+
     info = (*vfs->open) (vfs, file, flags, mode);	/* open must be supported */
     free (file);
     if (!info){
@@ -354,7 +434,7 @@ int mc_open (char *file, int flags, ...)
 	result = vfs->name ? (*vfs->name)callarg : -1; \
         post \
 	if (result == -1) \
-	    errno = vfs->name ? ferrno (vfs) : EOPNOTSUPP; \
+	    errno = vfs->name ? ferrno (vfs) : E_NOTSUPP; \
 	return result; \
 	}
 
@@ -365,7 +445,8 @@ int mc_open (char *file, int flags, ...)
 
 MC_HANDLEOP(read, (int handle, char *buffer, int count), (vfs_info (handle), buffer, count) );
 
-int mc_ctl (int handle, int ctlop, int arg)
+int
+mc_ctl (int handle, int ctlop, int arg)
 {
     vfs *vfs;
     int result;
@@ -375,11 +456,14 @@ int mc_ctl (int handle, int ctlop, int arg)
     return result;
 }
 
-int mc_setctl (char *path, int ctlop, char *arg)
+int
+mc_setctl (char *path, int ctlop, char *arg)
 {
     vfs *vfs;
     int result;
 
+    if (!path)
+	vfs_die( "You don't want to pass NULL to mc_setctl." );
     path = vfs_canon (path);
     vfs = vfs_type (path);    
     result = vfs->setctl ? (*vfs->setctl)(vfs, path, ctlop, arg) : 0;
@@ -387,7 +471,8 @@ int mc_setctl (char *path, int ctlop, char *arg)
     return result;
 }
 
-int mc_close (int handle)
+int
+mc_close (int handle)
 {
     vfs *vfs;
     int result;
@@ -400,7 +485,7 @@ int mc_close (int handle)
 	return close (handle);
 
     if (!vfs->close)
-        vfs_die( "VFS must support close.\n" );
+        vfs_die ("VFS must support close.\n");
     result = (*vfs->close)(vfs_info (handle));
     vfs_free_bucket (handle);
     if (result == -1)
@@ -409,7 +494,8 @@ int mc_close (int handle)
     return result;
 }
 
-DIR *mc_opendir (char *dirname)
+DIR *
+mc_opendir (char *dirname)
 {
     int  handle, *handlep;
     void *info;
@@ -432,7 +518,7 @@ DIR *mc_opendir (char *dirname)
     if (p)
         free (p);
     if (!info){
-        errno = vfs->opendir ? ferrno (vfs) : EOPNOTSUPP;
+        errno = vfs->opendir ? ferrno (vfs) : E_NOTSUPP;
 	return NULL;
     }
     handle = get_bucket ();
@@ -447,7 +533,8 @@ DIR *mc_opendir (char *dirname)
 /* This should strip the non needed part of a path name */
 #define vfs_name(x) x
 
-void mc_seekdir (DIR *dirp, int offset)
+void
+mc_seekdir (DIR *dirp, int offset)
 {
     int handle;
     vfs *vfs;
@@ -461,7 +548,7 @@ void mc_seekdir (DIR *dirp, int offset)
     if (vfs->seekdir)
         (*vfs->seekdir) (vfs_info (handle), offset);
     else
-        errno = EOPNOTSUPP;
+        errno = E_NOTSUPP;
 }
 
 #define MC_DIROP(name, type, onerr ) \
@@ -479,14 +566,15 @@ type mc_##name (DIR *dirp) \
     vfs = vfs_op (handle); \
     result = vfs->name ? (*vfs->name) (vfs_info (handle)) : onerr; \
     if (result == onerr) \
-        errno = vfs->name ? ferrno(vfs) : EOPNOTSUPP; \
+        errno = vfs->name ? ferrno(vfs) : E_NOTSUPP; \
     return result; \
 }
 
 MC_DIROP (readdir, struct dirent *, NULL)
 MC_DIROP (telldir, int, -1)
 
-int mc_closedir (DIR *dirp)
+int
+mc_closedir (DIR *dirp)
 {
     int handle = *(int *) dirp;
     vfs *vfs = vfs_op (handle);
@@ -506,7 +594,8 @@ MC_HANDLEOP (fstat, (int handle, struct stat *buf), (vfs_info (handle), buf))
  * You must strdup whatever this function returns, static buffers are in use
  */
 
-char *mc_return_cwd (void)
+char *
+mc_return_cwd (void)
 {
     char *p;
     struct stat my_stat, my_stat2;
@@ -532,12 +621,14 @@ char *mc_return_cwd (void)
     return current_dir;
 }
 
-char *mc_get_current_wd (char *buffer, int size)
+char *
+mc_get_current_wd (char *buffer, int size)
 {
     char *cwd = mc_return_cwd();
-    if (strlen (cwd) > size){
+    
+    if (strlen (cwd) > size)
       vfs_die ("Current_dir size overflow.\n");
-    }
+
     strcpy (buffer, cwd);
     return buffer;
 }
@@ -569,7 +660,7 @@ int mc_##name (char *name1, char *name2) \
     free (name1); \
     free (name2); \
     if (result == -1) \
-        errno = vfs->name ? ferrno (vfs) : EOPNOTSUPP; \
+        errno = vfs->name ? ferrno (vfs) : E_NOTSUPP; \
     return result; \
 }
 
@@ -589,7 +680,7 @@ off_t mc_lseek (int fd, off_t offset, int whence)
     vfs = vfs_op (fd);
     result = vfs->lseek ? (*vfs->lseek)(vfs_info (fd), offset, whence) : -1;
     if (result == -1)
-        errno = vfs->lseek ? ferrno (vfs) : EOPNOTSUPP;
+        errno = vfs->lseek ? ferrno (vfs) : E_NOTSUPP;
     return result;
 }
 
@@ -599,53 +690,13 @@ off_t mc_lseek (int fd, off_t offset, int whence)
 
 #define ISSLASH(a) (!a || (a == '/'))
 
-void
-vfs_kill_dots (char *path, char *local)
+char *
+vfs_canon (char *path)
 {
-    char *s = path, *t = local;
-    char *last_slash = path;
-    
-    *t++ = *s++;
-    while (1){
-	if (ISSLASH (*s)){
-	    if (*last_slash != '/')
-		vfs_die ("/ not there.\n");
-	    
-	    if (ISSLASH (last_slash [1]))
-		t--;
-	    
-	    if (last_slash [1] == '.'){
-		if (ISSLASH(last_slash [2]))
-		    t -= 2;
-		if ((last_slash [2] == '.') && (ISSLASH (last_slash [3]))){
-		    t -= 4;
-		    while ((t >= local) && (*t != '/'))
-			t--;
-		}
-	    }
-	    last_slash = s;
-	}
-	
-	if (t <= local){
-	    *local = '/';
-	    t = local;
-	}
-	if (!*s)
-	    break;
-	*t++ = *s++;
-	*t = 0;
-    }	
-    *t = 0;
-    if (!(*local)){
-	local [0] = '/';
-	local [1] = 0;
-    }
-}
+    if (!path) vfs_die("Can not canonize NULL");
 
-
-char *vfs_canon (char *path)
-{
-    if (*path == '~'){ /* Tilde expansion */
+    /* Tilde expansion */
+    if (*path == '~'){ 
     	char *local, *result;
 
     	local = tilde_expand (path);
@@ -657,7 +708,8 @@ char *vfs_canon (char *path)
 	    return strdup (path);
     }
 
-    if (*path != '/'){ /* Relative to current directory */
+    /* Relative to current directory */
+    if (*path != '/'){ 
     	char *local, *result;
 
 	if (current_dir [strlen (current_dir) - 1] == '/')
@@ -669,15 +721,20 @@ char *vfs_canon (char *path)
 	free (local);
 	return result;
     }
-    /* So we have path of following form:
+
+    /*
+     * So we have path of following form:
      * /p1/p2#op/.././././p3#op/p4. Good luck.
      */
+    mad_check( "(pre-canonicalize)", 0);
     canonicalize_pathname (path);
+    mad_check( "(post-canonicalize)", 0);
 
     return strdup (path);
 }
 
-vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
+vfsid
+vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
 {
     vfsid nvfsid;
     int freeit = 0;
@@ -685,6 +742,7 @@ vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
     if (dir [strlen (dir) - 1] != '/'){
         dir = copy_strings (dir, "/", NULL);    
         freeit = 1;
+
     }
     nvfsid = (*nvfs->getid)(nvfs, dir, par);
     if (freeit)
@@ -692,16 +750,20 @@ vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
     return nvfsid;
 }
 
-static int is_parent (vfs * nvfs, vfsid nvfsid, struct vfs_stamping *parent)
+static int
+is_parent (vfs * nvfs, vfsid nvfsid, struct vfs_stamping *parent)
 {
     struct vfs_stamping *stamp;
+
     for (stamp = parent; stamp; stamp = stamp->parent)
 	if (stamp->v == nvfs && stamp->id == nvfsid)
 	    break;
+
     return (stamp ? 1 : 0);
 }
 
-void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stamping *parent)
+void
+vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stamping *parent)
 {
 #ifndef VFS_STANDALONE
     vfs *nvfs, *n2vfs, *n3vfs;
@@ -730,7 +792,7 @@ void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stampin
 
     f = is_parent (oldvfs, oldvfsid, par);
     vfs_rm_parents (par);
-    if ((nvfs == oldvfs && nvfs == oldvfsid) || oldvfsid == (vfsid *)-1 || f){
+    if ((nvfs == oldvfs && nvfsid == oldvfsid) || oldvfsid == (vfsid *)-1 || f){
 	return;
     }
 
@@ -759,7 +821,7 @@ void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stampin
     }
     
     if ((*oldvfs->nothingisopen) (oldvfsid)){
-	if (oldvfs == &extfs_vfs_ops && ((extfs_archive *) oldvfsid)->name == 0){
+	if (oldvfs == &vfs_extfs_ops && ((extfs_archive *) oldvfsid)->name == 0){
 	    /* Free the resources immediatly when we leave a mtools fs
 	       ('cd a:') instead of waiting for the vfs-timeout */
 	    (oldvfs->free) (oldvfsid);
@@ -772,7 +834,7 @@ void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stampin
 		stamp->id == (vfsid) - 1 ||
 		!(*stamp->v->nothingisopen) (stamp->id))
 		break;
-	    if (stamp->v == &extfs_vfs_ops && ((extfs_archive *) stamp->id)->name == 0){
+	    if (stamp->v == &vfs_extfs_ops && ((extfs_archive *) stamp->id)->name == 0){
 		(stamp->v->free) (stamp->id);
 		vfs_rmstamp (stamp->v, stamp->id, 0);
 	    } else
@@ -784,7 +846,8 @@ void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stampin
 #endif
 }
 
-static void vfs_stamp_path (char *path)
+static void
+vfs_stamp_path (char *path)
 {
     vfs *vfs;
     vfsid id;
@@ -800,7 +863,8 @@ static void vfs_stamp_path (char *path)
 }
 
 #ifndef VFS_STANDALONE
-void vfs_add_current_stamps (void)
+void
+vfs_add_current_stamps (void)
 {
     vfs_stamp_path (current_dir);
     if (get_current_type () == view_listing)
@@ -811,7 +875,8 @@ void vfs_add_current_stamps (void)
 #endif
 
 /* This function is really broken */
-int mc_chdir (char *path)
+int
+mc_chdir (char *path)
 {
     char *a, *b;
     int result;
@@ -858,32 +923,40 @@ int mc_chdir (char *path)
     return result;
 }
 
-int vfs_current_is_local (void)
+int
+vfs_current_is_local (void)
 {
-    return current_vfs == &local_vfs_ops;
+    return current_vfs == &vfs_local_ops;
 }
 
-int vfs_current_is_extfs (void)
+#if 0
+/* External world should not do differences between VFS-s */
+int
+vfs_current_is_extfs (void)
 {
-    return current_vfs == &extfs_vfs_ops;
+    return current_vfs == &vfs_extfs_ops;
 }
 
-int vfs_current_is_tarfs (void)
+int
+vfs_current_is_tarfs (void)
 {
-    return current_vfs == &tarfs_vfs_ops;
+    return current_vfs == &vfs_tarfs_ops;
 }
+#endif
 
-int vfs_file_is_local (char *filename)
+int
+vfs_file_is_local (char *filename)
 {
     vfs *vfs;
     
     filename = vfs_canon (filename);
     vfs = vfs_type (filename);
     free (filename);
-    return vfs == &local_vfs_ops;
+    return vfs == &vfs_local_ops;
 }
 
-int vfs_file_is_ftp (char *filename)
+int
+vfs_file_is_ftp (char *filename)
 {
 #ifdef USE_NETCODE
     vfs *vfs;
@@ -891,7 +964,7 @@ int vfs_file_is_ftp (char *filename)
     filename = vfs_canon (filename);
     vfs = vfs_type (filename);
     free (filename);
-    return vfs == &ftpfs_vfs_ops;
+    return vfs == &vfs_ftpfs_ops;
 #else
     return 0;
 #endif
@@ -902,7 +975,7 @@ char *vfs_get_current_dir (void)
     return current_dir;
 }
 
-void vfs_setup_wd (void)
+static void vfs_setup_wd (void)
 {
     current_dir = strdup ("/");
     if (!(vfs_flags & FL_NO_CWDSETUP))
@@ -919,7 +992,7 @@ MC_NAMEOP (rmdir, (char *path), (vfs, vfs_name (path)))
 MC_NAMEOP (mknod, (char *path, int mode, int dev), (vfs, vfs_name (path), mode, dev))
 
 #ifdef HAVE_MMAP
-struct mc_mmapping {
+static struct mc_mmapping {
     caddr_t addr;
     void *vfs_info;
     vfs *vfs;
@@ -951,7 +1024,8 @@ mc_mmap (caddr_t addr, size_t len, int prot, int flags, int fd, off_t offset)
     return result;
 }
 
-int mc_munmap (caddr_t addr, size_t len)
+int
+mc_munmap (caddr_t addr, size_t len)
 {
     struct mc_mmapping *mcm, *mcm2 = NULL;
     
@@ -972,7 +1046,8 @@ int mc_munmap (caddr_t addr, size_t len)
 
 #endif
 
-char *mc_def_getlocalcopy (vfs *vfs, char *filename)
+char *
+mc_def_getlocalcopy (vfs *vfs, char *filename)
 {
     char *tmp;
     int fdin, fdout, i;
@@ -982,13 +1057,13 @@ char *mc_def_getlocalcopy (vfs *vfs, char *filename)
     fdin = mc_open (filename, O_RDONLY);
     if (fdin == -1)
         return NULL;
-    tmp = tmpnam(NULL);
-    fdout = creat (tmp, 0600); 		/* FIXME: What about symlink attack ? */
+    tmp = tempnam (NULL, "mclocalcopy");
+    fdout = open (tmp, O_CREAT|O_WRONLY|O_TRUNC|O_EXCL, 0600);
     if (fdout == -1){
         mc_close (fdin);
+	free (tmp);
         return NULL;
     }
-    tmp = strdup (tmp);
     while ((i = mc_read (fdin, buffer, sizeof (buffer))) == sizeof (buffer)){
         write (fdout, buffer, i);
     }
@@ -1002,7 +1077,8 @@ char *mc_def_getlocalcopy (vfs *vfs, char *filename)
     return tmp;
 }
 
-char *mc_getlocalcopy (char *path)
+char *
+mc_getlocalcopy (char *path)
 {
     vfs *vfs;
     char *result;
@@ -1017,7 +1093,8 @@ char *mc_getlocalcopy (char *path)
     return result;
 }
 
-void mc_def_ungetlocalcopy (vfs *vfs, char *filename, char *local, int has_changed)
+void
+mc_def_ungetlocalcopy (vfs *vfs, char *filename, char *local, int has_changed)
 {
     if (has_changed){
         int fdin, fdout, i;
@@ -1048,7 +1125,8 @@ void mc_def_ungetlocalcopy (vfs *vfs, char *filename, char *local, int has_chang
     free (local);
 }
 
-void mc_ungetlocalcopy (char *path, char *local, int has_changed)
+void
+mc_ungetlocalcopy (char *path, char *local, int has_changed)
 {
     vfs *vfs;
 
@@ -1062,19 +1140,22 @@ void mc_ungetlocalcopy (char *path, char *local, int has_changed)
 /*
  * Hmm, as timeout is minute or so, do we need to care about usecs?
  */
-inline int timeoutcmp (struct timeval *t1, struct timeval *t2)
+inline int
+timeoutcmp (struct timeval *t1, struct timeval *t2)
 {
     return ((t1->tv_sec < t2->tv_sec)
 	    || ((t1->tv_sec == t2->tv_sec) && (t1->tv_usec <= t2->tv_usec)));
 }
 
-void vfs_timeout_handler (void)
+void
+vfs_timeout_handler (void)
 {
     struct timeval time;
     struct vfs_stamping *stamp, *st;
 
     gettimeofday (&time, NULL);
     time.tv_sec -= vfs_timeout;
+
     for (stamp = stamps; stamp != NULL;){
         if (timeoutcmp (&stamp->time, &time)){
             st = stamp->next;
@@ -1086,7 +1167,8 @@ void vfs_timeout_handler (void)
     }
 }
 
-void vfs_init (void)
+void
+vfs_init (void)
 {
     time_t current_time;
     struct tm *t;
@@ -1098,27 +1180,28 @@ void vfs_init (void)
     current_mon  = t->tm_mon;
     current_year = t->tm_year;
 
-    /* We do not want to register local_vfs_ops */
+    /* We do not want to register vfs_local_ops */
 
 #ifdef USE_NETCODE
     tcp_init();
-    vfs_register (&ftpfs_vfs_ops);
-    vfs_register (&mcfs_vfs_ops);
+    vfs_register (&vfs_ftpfs_ops);
+    vfs_register (&vfs_mcfs_ops);
 #endif
 
-    vfs_register (&fish_vfs_ops);
-    vfs_register (&extfs_vfs_ops);
-    vfs_register (&sfs_vfs_ops);
-    vfs_register (&tarfs_vfs_ops);
+    vfs_register (&vfs_fish_ops);
+    vfs_register (&vfs_extfs_ops);
+    vfs_register (&vfs_sfs_ops);
+    vfs_register (&vfs_tarfs_ops);
 
 #ifdef USE_EXT2FSLIB
-    vfs_register (&undelfs_vfs_ops);
+    vfs_register (&vfs_undelfs_ops);
 #endif
 
     vfs_setup_wd ();
 }
 
-void vfs_free_resources (char *path)
+void
+vfs_free_resources (char *path)
 {
     vfs *vfs;
     vfsid vid;
@@ -1133,7 +1216,8 @@ void vfs_free_resources (char *path)
 
 #if 0
 /* Shutdown a vfs given a path name */
-void vfs_shut_path (char *p)
+void
+vfs_shut_path (char *p)
 {
     vfs *the_vfs;
     struct vfs_stamping *par;
@@ -1145,7 +1229,8 @@ void vfs_shut_path (char *p)
 }
 #endif
 
-void vfs_shut (void)
+void
+vfs_shut (void)
 {
     struct vfs_stamping *stamp, *st;
     vfs *vfs;
@@ -1163,18 +1248,21 @@ void vfs_shut (void)
     if (current_dir)
 	free (current_dir);
 
-    FOR_ALL_VFS
+    for (vfs=vfs_list; vfs; vfs=vfs->next)
         if (vfs->done)
 	     (*vfs->done) (vfs);
 }
 
-/* These ones grab information from the VFS
+/*
+ * These ones grab information from the VFS
  *  and handles them to an upper layer
  */
-void vfs_fill_names (void (*func)(char *))
+void
+vfs_fill_names (void (*func)(char *))
 {
     vfs *vfs;
-    FOR_ALL_VFS
+
+    for (vfs=vfs_list; vfs; vfs=vfs->next)
         if (vfs->fill_names)
 	    (*vfs->fill_names) (vfs, func);
 }
@@ -1185,7 +1273,8 @@ void vfs_fill_names (void (*func)(char *))
 static char *columns [MAXCOLS];	/* Points to the string in column n */
 static int   column_ptr [MAXCOLS]; /* Index from 0 to the starting positions of the columns */
 
-int split_text (char *p)
+int
+vfs_split_text (char *p)
 {
     char *original = p;
     int  numcols;
@@ -1204,14 +1293,16 @@ int split_text (char *p)
     return numcols;
 }
 
-static int is_num (int idx)
+static int
+is_num (int idx)
 {
     if (!columns [idx] || columns [idx][0] < '0' || columns [idx][0] > '9')
 	return 0;
     return 1;
 }
 
-static int is_time (char *str, struct tm *tim)
+static int
+is_time (char *str, struct tm *tim)
 {
     char *p, *p2;
 
@@ -1233,28 +1324,33 @@ static int is_time (char *str, struct tm *tim)
 
 static int is_year(char *str, struct tm *tim)
 {
-  /* Old code recognized 02904baa as year identification :-( */
     long year;
 
     if (strchr(str,':'))
         return 0;
+
     if (strlen(str)!=4)
         return 0;
+
     if (sscanf(str, "%ld", &year) != 1)
         return 0;
+
     if (year < 1900 || year > 3000)
         return 0;
+
     tim->tm_year = (int) (year - 1900);
+
     return 1;
 }
 
 /*
  * FIXME: this is broken. Consider following entry:
--rwx------   1 root     root            1 Aug 31 10:04 2904 1234
-where "2904 1234" is filename. Well, this code decodes it as year :-(.
+ * -rwx------   1 root     root            1 Aug 31 10:04 2904 1234
+ * where "2904 1234" is filename. Well, this code decodes it as year :-(.
  */
 
-int parse_filetype (char c)
+int
+vfs_parse_filetype (char c)
 {
     switch (c){
         case 'd': return S_IFDIR; 
@@ -1272,7 +1368,7 @@ int parse_filetype (char c)
     }
 }
 
-int parse_filemode (char *p)
+int vfs_parse_filemode (char *p)
 {	/* converts rw-rw-rw- into 0666 */
     int res = 0;
     switch (*(p++)){
@@ -1329,7 +1425,7 @@ int parse_filemode (char *p)
     return res;
 }
 
-int parse_filedate(int idx, time_t *t)
+int vfs_parse_filedate(int idx, time_t *t)
 {	/* This thing parses from idx in columns[] array */
     static char *month = "JanFebMarAprMayJunJulAugSepOctNovDec";
     static char *week = "SunMonTueWedThuFriSat";
@@ -1406,8 +1502,8 @@ int parse_filedate(int idx, time_t *t)
     return idx;
 }
 
-#define free_and_return(x) { free (p_copy); return (x); }
-int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
+int
+vfs_parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 {
     int idx, idx2, num_cols, isconc = 0;
     int i;
@@ -1416,13 +1512,14 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
     if (strncmp (p, "total", 5) == 0)
         return 0;
 
-    if ((i = parse_filetype(*(p++))) == -1)
-        return 0;
+    p_copy = strdup(p);
+    if ((i = vfs_parse_filetype(*(p++))) == -1)
+        goto error;
 
     s->st_mode = i;
     if (*p == '['){
 	if (strlen (p) <= 8 || p [8] != ']')
-	    return 0;
+	    goto error;
 	/* Should parse here the Notwell permissions :) */
 	if (S_ISDIR (s->st_mode))
 	    s->st_mode |= (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IXUSR | S_IXGRP | S_IXOTH);
@@ -1430,18 +1527,23 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	    s->st_mode |= (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR);
 	p += 9;
     } else {
-	if ((i = parse_filemode(p)) ==-1)
-	    return 0;
+	if ((i = vfs_parse_filemode(p)) == -1)
+	    goto error;
         s->st_mode |= i;
 	p += 9;
+
+        /* This is for an extra ACL attribute (HP-UX) */
+        if (*p == '+')
+            p++;
     }
-	
-    p_copy = strdup (p);
-    num_cols = split_text (p);
+
+    free(p_copy);
+    p_copy = strdup(p);
+    num_cols = vfs_split_text (p);
 
     s->st_nlink = atol (columns [0]);
     if (s->st_nlink <= 0)
-        free_and_return (0);
+        goto error;
 
     if (!is_num (1))
 	s->st_uid = finduid (columns [1]);
@@ -1455,7 +1557,7 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
             columns [idx][2] == '-' && columns [idx][5] == '-'))
             break;
     if (idx == 6 || (idx == 5 && !S_ISCHR (s->st_mode) && !S_ISBLK (s->st_mode)))
-        free_and_return (0);
+        goto error;
     if (idx < 5){
         char *p = strchr(columns [idx - 1], ',');
         if (p && p[1] >= '0' && p[1] <= '9')
@@ -1474,19 +1576,19 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
     if (S_ISCHR (s->st_mode) || S_ISBLK (s->st_mode)){
         char *p;
 	if (!is_num (idx2))
-	    free_and_return (0);
+	    goto error;
 #ifdef HAVE_ST_RDEV
 	s->st_rdev = (atol (columns [idx2]) & 0xff) << 8;
 #endif
 	if (isconc){
 	    p = strchr (columns [idx2], ',');
 	    if (!p || p [1] < '0' || p [1] > '9')
-	        free_and_return (0);
+	        goto error;
 	    p++;
 	} else {
 	    p = columns [idx2 + 1];
 	    if (!is_num (idx2+1))
-	        free_and_return (0);
+	        goto error;
 	}
 	
 #ifdef HAVE_ST_RDEV
@@ -1495,7 +1597,7 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	s->st_size = 0;
     } else {
 	if (!is_num (idx2))
-	    free_and_return (0);
+	    goto error;
 	
 	s->st_size = (size_t) atol (columns [idx2]);
 #ifdef HAVE_ST_RDEV
@@ -1503,9 +1605,9 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 #endif
     }
 
-    idx = parse_filedate(idx, &s->st_mtime);
+    idx = vfs_parse_filedate(idx, &s->st_mtime);
     if (!idx)
-        free_and_return (0);
+        goto error;
     /* Use resulting time value */
     s->st_atime = s->st_ctime = s->st_mtime;
     s->st_dev = 0;
@@ -1569,13 +1671,14 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	if (linkname)
 	    *linkname = NULL;
     }
-    free_and_return (1);
-}
+    free (p_copy);
+    return 1;
 
-void
-vfs_force_expire (char *pathname)
-{
-    mc_setctl( pathname, MCCTL_FORGET_ABOUT, NULL );
+error:
+    message_1s (1, "Could not parse:", p_copy);
+    if (p_copy != p)		/* Carefull! */
+	free (p_copy);
+    return 0;
 }
 
 void
@@ -1586,7 +1689,7 @@ vfs_die (char *m)
 }
 
 void
-print_vfs_stats (char *fs_name, char *action, char *file_name, int have, int need)
+vfs_print_stats (char *fs_name, char *action, char *file_name, int have, int need)
 {
     if (need) 
         print_vfs_message ("%s: %s: %s %3d%% (%ld bytes transfered)", 
@@ -1603,3 +1706,16 @@ vfs_get_password (char *msg)
     return (char *) input_dialog (msg, _("Password:"), "");
 }
 #endif
+
+/*
+ * Returns vfs path corresponding to given url. If passed string is
+ * not recognized as url, strdup(url) is returned.
+ */
+char *
+vfs_translate_url (char *url)
+{
+    if (strncmp (url, "ftp://", 6) == 0)
+        return copy_strings ("/#ftp:", url + 6, 0);
+    else
+        return strdup (url);
+}
